body { margin:0; padding:0; font: 12px Times, "Times New Roman"; text-align: center }
img { border:0 }

a { color: #666; text-decoration: none; border-bottom: solid 1px #ccc; }
a:hover { color: #333; border-bottom: solid 1px #aaa; }

acronym { border-bottom: dotted 1px #ccc; cursor: help; }

pre { font: 10px Courier, "Courier New"; background: #111; color: #fff; padding: 3px; border: solid 1px #ccc; }
hr { background: url(/thin/images/split.gif) no-repeat; height: 30px; width: 100px; border: 0; margin: 40px auto; }

input, select, textarea { font: 9px Tahoma, Arial; }
label { font-size: 11px; }

h1, h2, h3, h4 { margin: 12px 0 8px 0; padding: 0; }
h3 { font-size: 14px; }
h4 { font-size: 13px; }

ul#menu { padding: 4px; margin:0; background: #111; list-style: none; border-bottom: solid 1px #ccc; }
ul#menu li { margin:0; display: inline; }
ul#menu li a { color: #fff; text-decoration: none; padding: 6px; border: 0; }
ul#menu li a:hover { text-decoration: underline; }

#container { width: 300px; margin: 0 auto; }

#header { padding: 30px 0 10px 0; }
#header #logo { padding-bottom: 10px; }
#header #tag_line { margin: 4px 0; font-size: 12px; letter-spacing: -1px; color: #333; }

#content { line-height: 16px; text-align: left; }
#content h1 { margin: 40px 0 10px 0; padding: 0; text-align: center; }
#content h2 { margin: 40px 0 10px 0; padding: 0; text-align: center; }
#content h3 { margin: 40px 0 10px 0; padding: 0; text-align: center; }

#content ul { list-style-image: url(/thin/images/bullet.gif); margin: 0; padding: 0 20px; }
#content li { padding: 1px 0; }

#footer { margin: 60px 0 20px 0; font-size: 10px; color: #666; }

#sidebar { position: absolute; right: 0; width: 400px; text-align: right; padding: 10px; }

ul.list { list-style: none; }
ul.list li { margin: 0; padding: 1px 0; }

#content div.graph h3 { margin: 6px 0; }
#content div.graph { text-align: center; }

#content em.filename { display: block; text-align: right; margin-bottom: -8px; }

.clear { clear: both; }

/*RDoc*/
.dyn-source { display: none; }
.path { text-align: center; font: 9px Tahoma, Arial; }
.method-type { font: 9px Tahoma, Arial; color: #fff; background: #111; border: solid 1px #ccc; padding: 1px; }
